# -*- coding: utf-8 -*-
# modjkapi - API for modjk management
# Copyright © 2009-2011, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Higher level containment classes.
"""

from modjkapi.errors import JKCriteriaDoesNotExistError


class ObjectContainer(object):
    """
    Base class for any container of objects.
    """

    def __init__(self):
        """
        Creates an instance of the ObjectContainer class.
        """
        self._objects = []

    def get_attribute_from_objects(self, attr):
        """
        Returns back a single attribute from a set of objects.

        :Parameters:
           - `attr`: the attribute to get.
        """
        results = []
        for obj in self.objects():
            try:
                results.append(getattr(obj, attr))
            except:
                raise JKCriteriaDoesNotExistError(
                           "%s does not exist in %s." % (attr, type(obj)))
        return results

    def filter_by(self, **kwargs):
        """
        Queries objects for specific properties.

        :Parameters:
           - `kwargs`: all param=value for the query.
        """
        results = []
        for key in kwargs.keys():
            for obj in self.objects():
                try:
                    if getattr(obj, key) == kwargs[key]:
                        results.append(obj)
                except:
                    raise JKCriteriaDoesNotExistError(
                              "%s does not exist in %s." % (key, type(obj)))
        return results

    def get_one(self, **kwargs):
        """
        Queries objects for specific properties and returns the first that
        meets the requirement.

        :Parameters:
           - `kwargs`: all param=value for the query.
        """
        for key in kwargs.keys():
            for obj in self.objects():
                try:
                    if getattr(obj, key) == kwargs[key]:
                        return obj
                except:
                    raise JKCriteriaDoesNotExistError(
                              "%s does not exist in %s." % (key, type(obj)))
        raise JKQueryHasNoResultsError(
                  'Could not find matches for given criteria')

    def objects(self):
        """
        Returns the objects.
        """
        return self._objects
