# -*- coding: utf-8 -*-
# modjkapi - API for modjk management
# Copyright © 2009-2011, Red Hat, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
Shortcuts for modjkapi.
"""

__docformat__ = 'restructuredtext'


import types
import xml.sax

from StringIO import StringIO
from time import sleep

from modjkapi import (JKManagerHandler,
                      JKManagerWorkerObjectFactory,
                      JKManagerBalancerObjectFactory)
from modjkapi.errors import MaxTriesAttemptedError


def get_worker_when(worker_name, jkmanager_xml_url, param,
                    value, seconds, max_tries=None):
    """
    Polls and recreates the objects every X seconds.

    :Parameters:
       - `worker_name`: the name of the worker to wait on.
       - `jkmanager_xml_url`: the url to snag the xml from.
       - `param`: The parameter to watch.
       - `value`: The value to look for.
       - `seconds`: the amount of seconds between each poll.
       - `max_tries`: optional max amount of polls to do.
    """
    counter = 0

    # Go on 'forever'
    while True:
        factory = JKManagerWorkerObjectFactory(jkmanager_xml_url)
        # Find out object ...
        for worker in factory.objects():
            if worker.name == worker_name:
                # Check to see if we are busy .. if not return the worker
                if getattr(worker, param) == value:
                    return worker
        # Keep count of how many loops we make
        counter += 1
        # If we have max tries and we meet or exceed then stop
        if max_tries:
            if counter >= max_tries:
                raise MaxTriesAttempted(
                          "%s did not meet clause after %s attempts." % (
                              worker_name, counter))
        # Sleep for the requested seconds
        sleep(seconds)


def get_manage_data_string(jkmanager_xml_url):
    """
    Returns a formatted string from the jkmanager url.

    :Parameters:
       - `jkmanager_xml_url`: the url to snag the xml from.
    """
    result_io = StringIO()
    factory = JKManagerBalancerObjectFactory(jkmanager_xml_url)

    for balancer in factory.objects():
        result_io.write("Balancer: %s\n" % balancer.name)
        for worker in balancer.workers:
            result_io.write("\tworker: %s\n" % worker.name)
            # Be very specific about what we want to see ...
            for item in dir(worker):
                # avoid stuff we set
                if item not in ['name', 'meta', 'balancer']:
                    # no callables and no _'s
                    if '_' != item[:1] and not callable(getattr(worker, item)):
                        # Only general types
                        if type(item) in [types.IntType,
                                          types.StringType,
                                          types.UnicodeType]:
                            result_io.write(
                                "\t\t%s: %s\n" % (item, getattr(worker, item)))
    result_io.flush()
    result_io.seek(0)

    return result_io.read()
